@@ -34,6 +34,7 @@ import glob
 
 # purely to initialize the netstatus
 import softwarecenter.netstatus
+from softwarecenter.netstatus import network_state_is_connected
 
 from SimpleGtkbuilderApp import SimpleGtkbuilderApp
 from softwarecenter.db.application import Application, DebFileApplication
@@ -258,6 +259,10 @@ class SoftwareCenterApp(SimpleGtkbuilderApp):
                                    self.on_view_switcher_changed)
         self.view_switcher.width = self.scrolledwindow_viewswitcher.get_property('width-request')
         self.view_switcher.connect('size-allocate', self.on_viewswitcher_resized)
+        
+        # expand the Get Software node in the viewswitcher by default so that its important subitems
+        # (e.g., For Purchase and Independent) are always clearly visible and available
+        self.view_switcher.expand_available_node()
 
         # launchpad integration help, its ok if that fails
         try:
@@ -362,7 +367,7 @@ class SoftwareCenterApp(SimpleGtkbuilderApp):
         #       support in aptdaemon (see LP: #723911)
         file_menu = self.builder.get_object("menu1")
         file_menu.remove(self.builder.get_object("menuitem_deauthorize_computer"))
-            
+
     # helper
     def _rebuild_and_reopen_local_db(self, pathname):
         """ helper that rebuilds a db and reopens it """
@@ -394,7 +399,10 @@ class SoftwareCenterApp(SimpleGtkbuilderApp):
     
     def on_channel_pane_created(self, widget):
         channel_section = SoftwareSection()
-        channel_section.set_image(VIEW_PAGE_CHANNEL, os.path.join(self.datadir, 'images/arrows.png'))
+        # for background rendering, channel panes can specify either available or installed id's
+        # we need only set the installed case here as the available case has been set previously
+        # during the available_pane setup (LP: #724724)
+        channel_section.set_image(VIEW_PAGE_INSTALLED, os.path.join(self.datadir, 'images/arrows.png'))
         channel_section.set_color('#aea79f')
         self.channel_pane.set_section(channel_section)
 
@@ -600,6 +608,7 @@ class SoftwareCenterApp(SimpleGtkbuilderApp):
         # update menu items
         pkg_state = None
         error = None
+        is_network_available = network_state_is_connected()
         # FIXME:  Use a gtk.Action for the Install/Remove/Buy/Add Source/Update Now action
         #         so that all UI controls (menu item, applist view button and appdetails
         #         view button) are managed centrally:  button text, button sensitivity,
@@ -613,13 +622,13 @@ class SoftwareCenterApp(SimpleGtkbuilderApp):
             self.menuitem_install.set_sensitive(False)
             self.menuitem_remove.set_sensitive(False)
         elif pkg_state == PKG_STATE_UPGRADABLE or pkg_state == PKG_STATE_REINSTALLABLE and not error:
-            self.menuitem_install.set_sensitive(True)
+            self.menuitem_install.set_sensitive(is_network_available)
             self.menuitem_remove.set_sensitive(True)
         elif pkg_state == PKG_STATE_INSTALLED:
             self.menuitem_install.set_sensitive(False)
             self.menuitem_remove.set_sensitive(True)
         elif pkg_state == PKG_STATE_UNINSTALLED and not error:
-            self.menuitem_install.set_sensitive(True)
+            self.menuitem_install.set_sensitive(is_network_available)
             self.menuitem_remove.set_sensitive(False)
         elif (not pkg_state and 
               not self.active_pane.is_category_view_showing() and 
@@ -634,6 +643,7 @@ class SoftwareCenterApp(SimpleGtkbuilderApp):
             self.menuitem_install.set_sensitive(False)
             self.menuitem_remove.set_sensitive(False)
         # return False to ensure that a possible glib.timeout_add ends
+        self.menuitem_reinstall_purchases.set_sensitive(is_network_available)
         return False
 
     def on_menuitem_launchpad_private_ppas_activate(self, menuitem):
@@ -661,7 +671,9 @@ class SoftwareCenterApp(SimpleGtkbuilderApp):
     def _create_dbus_sso_if_needed(self):
         if not self.sso:
             from backend.login_sso import LoginBackendDbusSSO
-            appname = _("Ubuntu Software Center Store")
+            # see bug #773214 for the rational
+            #appname = _("Ubuntu Software Center Store")
+            appname = "Ubuntu Software Center Store"
             login_text = _("To reinstall previous purchases, sign in to the "
                            "Ubuntu Single Sign-On account you used to pay for them.")
             self.sso = LoginBackendDbusSSO(self.window_main.window.xid,
@@ -1145,9 +1157,6 @@ class SoftwareCenterApp(SimpleGtkbuilderApp):
         if (self.config.has_option("general", "maximized") and
             self.config.getboolean("general", "maximized")):
             self.window_main.maximize()
-        if (self.config.has_option("general", "available-node-expanded") and
-            self.config.getboolean("general", "available-node-expanded")):
-            self.view_switcher.expand_available_node()
         if (self.config.has_option("general", "installed-node-expanded") and
             self.config.getboolean("general", "installed-node-expanded")):
             self.view_switcher.expand_installed_node()
@@ -1170,11 +1179,6 @@ class SoftwareCenterApp(SimpleGtkbuilderApp):
             # size only matters when non-maximized
             size = self.window_main.get_size() 
             self.config.set("general","size", "%s, %s" % (size[0], size[1]))
-        available_node_expanded = self.view_switcher.is_available_node_expanded()
-        if available_node_expanded:
-            self.config.set("general", "available-node-expanded", "True")
-        else:
-            self.config.set("general", "available-node-expanded", "False")
         installed_node_expanded = self.view_switcher.is_installed_node_expanded()
         if installed_node_expanded:
             self.config.set("general", "installed-node-expanded", "True")
