@@ -26,12 +26,11 @@ import gio
 import glib
 import logging
 import simplejson
-import time
 import threading
 
 from softwarecenter.distro import get_distro
-from softwarecenter.enums import BUY_SOMETHING_HOST
-from softwarecenter.utils import get_current_arch
+from softwarecenter.enums import BUY_SOMETHING_HOST, BUY_SOMETHING_HOST_ANONYMOUS
+from softwarecenter.utils import get_current_arch, get_language
 
 # possible workaround for bug #599332 is to try to import lazr.restful
 # import lazr.restful
@@ -47,11 +46,24 @@ from Queue import Queue
 
 from login import LoginBackend
 
-UBUNTU_SSO_SERVICE = "https://login.ubuntu.com/api/1.0"
+UBUNTU_SSO_SERVICE = os.environ.get(
+    "USSOC_SERVICE_URL", "https://login.ubuntu.com/api/1.0")
 UBUNTU_SOFTWARE_CENTER_AGENT_SERVICE = BUY_SOMETHING_HOST+"/api/1.0"
 
-class EmptyObject(object):
-    pass
+class AttributesObject(object):
+    """ convinient object to hold attributes """
+    MAX_REPR_STRING_SIZE = 30
+
+    def __repr__(self):
+        s = "<'%s': " % self.__class__.__name__
+        for key in vars(self):
+            value = str(getattr(self, key))
+            if len(value) > self.MAX_REPR_STRING_SIZE:
+                value = "%s..." % value[:self.MAX_REPR_STRING_SIZE]
+            s += "%s='%s';" % (key, value)
+        s += ">"
+        return s
+
 
 def restful_collection_to_real_python(restful_list):
     """ take a restful and convert it to a python list with real python
@@ -59,7 +71,7 @@ def restful_collection_to_real_python(restful_list):
     """
     l = []
     for entry in restful_list:
-        o = EmptyObject()
+        o = AttributesObject()
         for attr in entry.lp_attributes:
             setattr(o, attr, getattr(entry, attr))
         l.append(o)
@@ -128,11 +140,62 @@ class RestfulClientWorker(threading.Thread):
                     result_callback(res)
                 self._pending_requests.task_done()
             # wait a bit
+            import time
             time.sleep(0.1)
             if (self._shutdown and
                 self._pending_requests.empty()):
                 return
 
+class UbuntuSSOAPI(gobject.GObject):
+
+    __gsignals__ = {
+        "whoami" : (gobject.SIGNAL_RUN_LAST,
+                    gobject.TYPE_NONE, 
+                    (gobject.TYPE_PYOBJECT,),
+                    ),
+        "error" : (gobject.SIGNAL_RUN_LAST,
+                    gobject.TYPE_NONE, 
+                    (gobject.TYPE_PYOBJECT,),
+                    ),
+
+        }
+       
+    def __init__(self, token):
+        gobject.GObject.__init__(self)
+        self._whoami = None
+        self._error = None
+        self.service = UBUNTU_SSO_SERVICE
+        self.token = token
+        token = OAuthToken(self.token["token"], self.token["token_secret"])
+        authorizer = OAuthAuthorizer(self.token["consumer_key"],
+                                     self.token["consumer_secret"],
+                                     access_token=token)
+        self.worker_thread =  RestfulClientWorker(authorizer, self.service)
+        self.worker_thread.start()
+        glib.timeout_add(200, self._monitor_thread)
+
+    def _monitor_thread(self):
+        # glib bit of the threading, runs in the main thread
+        if self._whoami is not None:
+            self.emit("whoami", self._whoami)
+            self._whoami = None
+        if self._error is not None:
+            self.emit("error", self._error)
+            self._error = None
+        return True
+
+    def _thread_whoami_done(self, result):
+        self._whoami = result
+
+    def _thread_whoami_error(self, e):
+        self._error = e
+
+    def whoami(self):
+        self.worker_thread.queue_request("accounts.me", (), {},
+                                         self._thread_whoami_done,
+                                         self._thread_whoami_error)
+
+
 class SoftwareCenterAgent(gobject.GObject):
 
     __gsignals__ = {
@@ -272,16 +335,16 @@ class UbuntuSSOlogin(LoginBackend):
 
     def _thread_authentication_done(self, result):
         # runs in the thread context, can not touch gui or glib
-        print "_authentication_done", result
+        #print "_authentication_done", result
         self._oauth_credentials = result
 
     def _thread_authentication_error(self, e):
         # runs in the thread context, can not touch gui or glib
-        print "_authentication_error", type(e)
+        #print "_authentication_error", type(e)
         self._login_failure = e
 
     def __del__(self):
-        print "del"
+        #print "del"
         if self.worker_thread:
             self.worker_thread.shutdown()
 
@@ -303,8 +366,9 @@ class SoftwareCenterAgentAnonymous(gobject.GObject):
                   ),
         }
     
-    def __init__(self):
+    def __init__(self, ignore_etag=False):
         gobject.GObject.__init__(self)
+        self.ignore_etag = ignore_etag
         self.distro = get_distro()
         self.log = logging.getLogger("softwarecenter.backend.scagent")
         # make sure we have the cachdir
@@ -317,7 +381,7 @@ class SoftwareCenterAgentAnonymous(gobject.GObject):
             for that host. If there is none, return a invalid etag (no
             quote) that will never match
         """
-        if os.path.exists(etagfile):
+        if os.path.exists(etagfile) and not self.ignore_etag:
             return open(etagfile).read()
         else:
             return "invalid-etag"
@@ -349,7 +413,7 @@ class SoftwareCenterAgentAnonymous(gobject.GObject):
         # all good, convert to real objects and emit available items
         items = []
         for item_dict in json_list:
-            o = EmptyObject()
+            o = AttributesObject()
             for (key, value) in item_dict.iteritems():
                 setattr(o, key, value)
             items.append(o)
@@ -372,19 +436,21 @@ class SoftwareCenterAgentAnonymous(gobject.GObject):
             self.log.debug("etags match (%s == %s), doing nothing" % (
                     etag, self.latest_etag))
             self.emit("available", [])
-    def query_available(self):
+    def query_available(self, include_approved_but_unpublished=False):
         """ query what software is available for the current codename/arch 
             Note that this function is async and emits "available" or "error"
             signals when done
         """
         series_name = self.distro.get_codename()
         arch_tag = get_current_arch()
-        # the server supports only english for now
-        lang = "en"
-        url = BUY_SOMETHING_HOST + "/apps/%(lang)s/ubuntu/%(series)s/%(arch)s" % {
+        lang = get_language()
+        url = BUY_SOMETHING_HOST_ANONYMOUS + "/apps/%(lang)s/ubuntu/%(series)s/%(arch)s" % {
             'lang' : lang,
             'series' : series_name,
             'arch' : arch_tag, }
+        # include not yet-published items (if asked for)
+        if include_approved_but_unpublished:
+            url += "?include_approved=True"
         # load latest etag if available
         self.latest_etag = self._load_etag(self.etagfile, url)
         f = gio.File(url)
@@ -410,9 +476,13 @@ def _available_for_me_result(scagent, result):
     print "_available_for_me: ", [x.package_name for x in result]
 
 def _available(scagent, result):
-    print "_available: ", [x.name for x in result]
+    print "_available"
+    for available in result:
+        print available.name, available.archive_id, available.archive_root
 def _error(scaagent, errormsg):
     print "_error:", errormsg
+def _whoami(sso, whoami):
+    print "whoami: ", whoami
 
 # interactive test code
 if __name__ == "__main__":
@@ -432,11 +502,22 @@ if __name__ == "__main__":
         scagent.query_available_for_me("dummy_oauth", "dummy openid")
 
     elif sys.argv[1] == "sso":
-        sso = UbuntuSSOlogin()
-        sso.connect("login-successful", _login_success)
-        sso.connect("login-failed", _login_failed)
-        sso.connect("need-username-password", _login_need_user_and_password)
-        sso.login()
+        def _dbus_maybe_login_successful(ssologin, oauth_result):
+            sso = UbuntuSSOAPI(oauth_result)
+            sso.connect("whoami", _whoami)
+            sso.connect("error", _error)
+            sso.whoami()
+        from login_sso import LoginBackendDbusSSO
+        backend = LoginBackendDbusSSO("", "appname", "login_text")
+        backend.connect("login-successful", _dbus_maybe_login_successful)
+        backend.login_or_register()
+
+    elif sys.argv[1] == "ssologin":
+        ssologin = UbuntuSSOlogin()
+        ssologin.connect("login-successful", _login_success)
+        ssologin.connect("login-failed", _login_failed)
+        ssologin.connect("need-username-password", _login_need_user_and_password)
+        ssologin.login()
         
     elif sys.argv[1] == "agent-anon":
         anon_agent = SoftwareCenterAgentAnonymous()
