@@ -122,7 +122,7 @@ class Review(object):
             self.id, self.review_text, self.reviewer_username)
     def __cmp__(self, other):
         # first compare version, high version number first
-        vc = version_compare(self.version, other.version)
+        vc = upstream_version_compare(self.version, other.version)
         if vc != 0:
             return vc
         # then usefulness
@@ -180,7 +180,7 @@ class ReviewLoader(object):
                 LOG.exception("review stats cache load failure")
                 os.rename(self.REVIEW_STATS_CACHE_FILE, self.REVIEW_STATS_CACHE_FILE+".fail")
 
-    def get_reviews(self, application, callback):
+    def get_reviews(self, application, callback, page=1):
         """run callback f(app, review_list) 
            with list of review objects for the given
            db.database.Application object
@@ -299,7 +299,7 @@ class ReviewLoader(object):
                     if str(review.id) == str(review_id):
                         # remove the one we don't want to see anymore
                         self._reviews[app].remove(review)
-                        callback(app, self._reviews[app])
+                        callback(app, self._reviews[app], None, 'remove', review)
                         break
 
     def _on_submit_usefulness_finished(self, pid, status, (review_id, is_useful, stdout_fd, callback)):
@@ -322,7 +322,7 @@ class ReviewLoader(object):
                         review.usefulness_total = getattr(review, "usefulness_total", 0) + 1
                         if is_useful:
                             review.usefulness_favorable = getattr(review, "usefulness_favorable", 0) + 1
-                        callback(app, self._reviews[app], useful_votes)
+                        callback(app, self._reviews[app], useful_votes, 'replace', review)
                         break
         else:
             LOG.debug("submit usefulness id=%s failed with exitcode %s" % (
@@ -331,7 +331,7 @@ class ReviewLoader(object):
                 for review in reviews:
                     if str(review.id) == str(review_id):
                         review.usefulness_submit_error = exitcode
-                        callback(app, self._reviews[app])
+                        callback(app, self._reviews[app], None, 'replace', review)
                         break
 
 
@@ -359,7 +359,7 @@ class ReviewLoaderSpawningRNRClient(ReviewLoader):
         self.rnrclient._offline_mode = not network_state_is_connected()
 
     # reviews
-    def get_reviews(self, translated_app, callback):
+    def get_reviews(self, translated_app, callback, page=1):
         """ public api, triggers fetching a review and calls callback
             when its ready
         """
@@ -391,24 +391,30 @@ class ReviewLoaderSpawningRNRClient(ReviewLoader):
                "--origin", origin, 
                "--distroseries", distroseries, 
                "--pkgname", str(app.pkgname), # ensure its str, not unicode
+               "--page", str(page),
               ]
         try:
             (pid, stdin, stdout, stderr) = glib.spawn_async(
                 cmd, flags = glib.SPAWN_DO_NOT_REAP_CHILD, 
                 standard_output=True, standard_error=True)
-            glib.child_watch_add(pid, self._reviews_loaded_watcher, data=(app, stdout, stderr, callback))
+            glib.child_watch_add(pid, self._reviews_helper_finished, data=(app, stdout, stderr, callback))
+            glib.io_add_watch(stdout, glib.IO_IN, self._reviews_io_ready, (app, stdout, callback))
         except Exception as e:
             raise Exception("failed to launch: '%s' (error: '%s')" % (cmd, e))
 
-    def _reviews_loaded_watcher(self, pid, status, (app, stdout, stderr, callback)):
+    def _reviews_helper_finished(self, pid, status, (app, stdout, stderr, callback)):
         """ watcher function in parent using glib """
         # get status code
         res = os.WEXITSTATUS(status)
+        if res != 0:
+            LOG.warn("exit code %s from helper" % res)
         # check stderr
         err = os.read(stderr, 4*1024)
         if err:
-            logging.warn("got error from helper: '%s'" % err)
+            LOG.warn("got error from helper: '%s'" % err)
         os.close(stderr)
+
+    def _reviews_io_ready(self, source, condition, (app, stdout, callback)):
         # read the raw data
         data = ""
         while True:
@@ -424,8 +430,9 @@ class ReviewLoaderSpawningRNRClient(ReviewLoader):
         for r in piston_reviews:
             reviews.append(Review.from_piston_mini_client(r))
         # add to our dicts and run callback
-        self._reviews[app] = reviews
+        self._reviews[app] = sorted(reviews, reverse=True)
         callback(app, self._reviews[app])
+        return False
 
     # stats
     def refresh_review_stats(self, callback):
@@ -452,19 +459,24 @@ class ReviewLoaderSpawningRNRClient(ReviewLoader):
         (pid, stdin, stdout, stderr) = glib.spawn_async(
             cmd, flags = glib.SPAWN_DO_NOT_REAP_CHILD, 
             standard_output=True, standard_error=True)
-        glib.child_watch_add(pid, self._review_stats_loaded_watcher, data=(stdout, stderr, callback))
+        glib.child_watch_add(pid, self._review_stats_helper_finished, data=(stdout, stderr, callback))
+        glib.io_add_watch(stdout, glib.IO_IN, self._review_stats_io_ready, (stdout, callback))
 
-    def _review_stats_loaded_watcher(self, pid, status, (stdout, stderr, callback)):
+    def _review_stats_helper_finished(self, pid, status, (stdout, stderr, callback)):
         """ waits for the process that gets the data
             to finish and emits callback then """
-        # shorthand
-        review_stats = self.REVIEW_STATS_CACHE
         res = os.WEXITSTATUS(status)
-        # check stderr first
+        if res != 0:
+            LOG.warn("exit code %s from helper" % res)
+        # log stderr
         err = os.read(stderr, 4*1024)
         if err:
-            logging.warn("got error from helper: '%s'" % err)
+            LOG.warn("got error from helper: '%s'" % err)
         os.close(stderr)
+
+    def _review_stats_io_ready(self, source, condition, (stdout, callback)):
+        """ process stdout from the helper """
+        review_stats = self.REVIEW_STATS_CACHE
         # get data
         data = ""
         while True:
@@ -480,10 +492,10 @@ class ReviewLoaderSpawningRNRClient(ReviewLoader):
             s.ratings_average = float(r.ratings_average)
             s.ratings_total = float(r.ratings_total)
             review_stats[s.app] = s
-
         self.REVIEW_STATS_CACHE = review_stats
         callback(review_stats)
         self.save_review_stats_cache_file()
+        return False
 
 class ReviewLoaderJsonAsync(ReviewLoader):
     """ get json (or gzip compressed json) """
@@ -506,9 +518,9 @@ class ReviewLoaderJsonAsync(ReviewLoader):
             review = Review.from_json(review_json)
             reviews.append(review)
         # run callback
-        callback(app, reviews)
+        callback(app, sorted(reviews, reverse=True))
 
-    def get_reviews(self, app, callback):
+    def get_reviews(self, app, callback, page=1):
         """ get a specific review and call callback when its available"""
         # FIXME: get this from the app details
         origin = self.cache.get_origin(app.pkgname)
@@ -579,7 +591,7 @@ class ReviewLoaderFake(ReviewLoader):
         return random.choice(self.LOREM.split("\n\n"))
     def _random_summary(self):
         return random.choice(self.SUMMARIES)
-    def get_reviews(self, application, callback):
+    def get_reviews(self, application, callback, page=1):
         if not application in self._review_stats_cache:
             self.get_review_stats(application)
         stats = self._review_stats_cache[application]
