@@ -20,6 +20,7 @@ import datetime
 import gettext
 import locale
 import re
+import urllib
 import subprocess
 
 from apt.utils import *
@@ -37,13 +38,21 @@ class Ubuntu(Distro):
     SCREENSHOT_LARGE_URL = "http://screenshots.ubuntu.com/screenshot-with-version/%(pkgname)s/%(version)s"
 
     # purchase subscription
-    PURCHASE_APP_URL = BUY_SOMETHING_HOST+"/subscriptions/en/ubuntu/maverick/+new/?%s"
+    PURCHASE_APP_URL = BUY_SOMETHING_HOST+"/subscriptions/%s/ubuntu/%s/+new/?%s"
+
+    # reviews
+    REVIEWS_SERVER = os.environ.get("SOFTWARE_CENTER_REVIEWS_HOST") or "http://reviews.ubuntu.com/reviews/api/1.0"
+    REVIEWS_URL = REVIEWS_SERVER + "/reviews/filter/%(language)s/%(origin)s/%(distroseries)s/%(version)s/%(pkgname)s%(appname)s/"
+
+    #REVIEW_STATS_URL = REVIEWS_SERVER+"/reviews/api/1.0/%(language)s/%(origin)s/%(distroseries)s/review-stats/"
+    # FIXME: does that make sense?!?
+    REVIEW_STATS_URL = REVIEWS_SERVER+"/review-stats"
 
     def get_app_name(self):
         return _("Ubuntu Software Center")
 
     def get_app_description(self):
-        return _("Lets you choose from thousands of free applications available for Ubuntu.")
+        return _("Lets you choose from thousands of applications available for Ubuntu.")
     
     def get_distro_channel_name(self):
         """ The name in the Release file """
@@ -55,7 +64,7 @@ class Ubuntu(Distro):
 
     def get_removal_warning_text(self, cache, pkg, appname, depends):
         primary = _("To remove %s, these items must be removed "
-                    "as well:" % appname)
+                    "as well:") % appname
         button_text = _("Remove All")
 
         # alter it if a meta-package is affected
@@ -94,9 +103,8 @@ class Ubuntu(Distro):
             return _("Proprietary")
 
     def is_supported(self, cache, doc, pkgname):
-        section = doc.get_value(XAPIAN_VALUE_ARCHIVE_SECTION)
-        if section == "main" and section == "restricted":
-            return True
+        # the doc does not by definition contain correct data regarding the
+        # section. Looking up in the cache seems just as fast/slow.
         if pkgname in cache and cache[pkgname].candidate:
             for origin in cache[pkgname].candidate.origins:
                 if (origin.origin == "Ubuntu" and 
@@ -206,7 +214,15 @@ class Ubuntu(Distro):
         """
         generates the url for a downloadable icon based on its pkgname and the icon filename itself
         """
-        full_archive_url = cache[pkgname].candidate.uri
+        if (cache is None or
+            not hasattr(cache._cache, '__contains__') or # cache not ready
+            not pkgname in cache):
+            return None
+        try:
+            full_archive_url = cache[pkgname].candidate.uri
+        except:
+            # it's ok if we can't get the icon url
+            return None
         split_at_pool = full_archive_url.split("pool")[0]
         # support ppas and extras.ubuntu.com
         if split_at_pool.endswith("/ppa/ubuntu/"):
