@@ -33,6 +33,9 @@ DB_SCHEMA_VERSION = "2"
 # the default limit for a search
 DEFAULT_SEARCH_LIMIT = 10000
 
+# the server size "page" for ratings&reviews
+REVIEWS_BATCH_PAGE_SIZE = 10
+
 # the various "views" that the app has
 VIEW_PAGE_AVAILABLE = "view-page-available"
 VIEW_PAGE_INSTALLED = "view-page-installed"
@@ -98,6 +101,7 @@ XAPIAN_VALUE_ICON_NEEDS_DOWNLOAD = 186
 XAPIAN_VALUE_THUMBNAIL_URL = 187
 XAPIAN_VALUE_SC_DESCRIPTION = 188
 XAPIAN_VALUE_APPNAME_UNTRANSLATED = 189
+XAPIAN_VALUE_ICON_URL = 190
 
 # fake channels
 PURCHASED_NEEDS_REINSTALL_MAGIC_CHANNEL_NAME = "for-pay-needs-reinstall"
