@@ -39,6 +39,8 @@ from enums import USER_AGENT, MISSING_APP_ICON, APP_ICON_SIZE, APP_INSTALL_PATH_
 
 from config import get_config
 
+from gettext import gettext as _
+
 # define additional entities for the unescape method, needed
 # because only '&amp;', '&lt;', and '&gt;' are included by default
 ESCAPE_ENTITIES = {"&apos;":"'",
@@ -249,6 +251,22 @@ def sources_filename_from_ppa_entry(entry):
     import apt_pkg
     name = "%s.list" % apt_pkg.URItoFileName(entry.uri)
     return name
+    
+def obfuscate_private_ppa_details(text):
+    """
+    hides any private PPA details that may be found in the given text
+    """
+    result = text
+    s = text.split()
+    for item in s:
+        if "private-ppa.launchpad.net" in item:
+            from urlparse import urlsplit
+            url_parts = urlsplit(item)
+            if url_parts.username:
+                result = result.replace(url_parts.username, "hidden")
+            if url_parts.password:
+                result = result.replace(url_parts.password, "hidden")
+    return result
 
 def release_filename_in_lists_from_deb_line(debline):
     """
@@ -307,15 +325,16 @@ def convert_desktop_file_to_installed_location(app_install_data_file_path, pkgna
         given app-install-data file path, and will also check directly for
         the desktop file that corresponds to a given pkgname.
     """
-    # "normal" case
-    installed_desktop_file_path = app_install_data_file_path.replace("app-install/desktop", "applications")
-    if os.path.exists(installed_desktop_file_path):
-        return installed_desktop_file_path  
-    # next, try case where a subdirectory is encoded in the app-install
-    # desktop filename, e.g. kde4_soundkonverter.desktop
-    installed_desktop_file_path = installed_desktop_file_path.replace(APP_INSTALL_PATH_DELIMITER, "/")
-    if os.path.exists(installed_desktop_file_path):
-        return installed_desktop_file_path
+    if app_install_data_file_path:
+        # "normal" case
+        installed_desktop_file_path = app_install_data_file_path.replace("app-install/desktop", "applications")
+        if os.path.exists(installed_desktop_file_path):
+            return installed_desktop_file_path  
+        # next, try case where a subdirectory is encoded in the app-install
+        # desktop filename, e.g. kde4_soundkonverter.desktop
+        installed_desktop_file_path = installed_desktop_file_path.replace(APP_INSTALL_PATH_DELIMITER, "/")
+        if os.path.exists(installed_desktop_file_path):
+            return installed_desktop_file_path
     # lastly, just try checking directly for the desktop file based on the pkgname itself
     if pkgname:
         installed_desktop_file_path =  "/usr/share/applications/%s.desktop" % pkgname
