@@ -733,6 +733,9 @@ class AppDetailsViewGtk(gtk.Viewport, AppDetailsViewBase):
         self.addons_to_install = self.addons_manager.addons_to_install
         self.addons_to_remove = self.addons_manager.addons_to_remove
 
+        # reviews
+        self._reviews_server_page = 1
+        
         # switches
         self._show_overlay = False
 
@@ -776,7 +779,23 @@ class AppDetailsViewGtk(gtk.Viewport, AppDetailsViewBase):
         else:
             self.review_stats_widget.hide()
 
-    def _reviews_ready_callback(self, app, reviews_data, my_votes=None):
+    def _on_more_reviews_clicked(self, uilist):
+        self._reviews_server_page += 1
+        self.reviews.show_spinner_with_message(_('Checking for reviews...'))
+        self.review_loader.get_reviews(
+            self.app, self._reviews_ready_callback,
+            page=self._reviews_server_page)
+    
+    def _review_update_single(self, action, review):
+        if action == 'replace':
+            self.reviews.replace_review(review)
+        elif action == 'remove':
+            self.reviews.remove_review(review)
+        return
+        
+
+    def _reviews_ready_callback(self, app, reviews_data, my_votes=None, 
+                                action=None, single_review=None):
         """ callback when new reviews are ready, cleans out the
             old ones
         """
@@ -809,12 +828,15 @@ class AppDetailsViewGtk(gtk.Viewport, AppDetailsViewBase):
         if my_votes:
             self.reviews.update_useful_votes(my_votes)
         
-        # FIXME: simplify UIReviewsList API to provide a single set_reviews()
-        # update the UI
-        self.reviews.clear()
-        for review in reviews_data:
-            self.reviews.add_review(review)
-        self.reviews.finished()
+        if action:
+            self._review_update_single(action, single_review)
+        else:
+            curr_list = self.reviews.get_all_review_ids()
+            for review in reviews_data:
+                if not review.id in curr_list:
+                    self.reviews.add_review(review)
+        
+        self.reviews.configure_reviews_ui()
 
     def on_test_drive_clicked(self, button):
         # weblive helpers
@@ -1098,6 +1120,7 @@ class AppDetailsViewGtk(gtk.Viewport, AppDetailsViewBase):
         self.reviews.connect("new-review", self._on_review_new)
         self.reviews.connect("report-abuse", self._on_review_report_abuse)
         self.reviews.connect("submit-usefulness", self._on_review_submit_usefulness)
+        self.reviews.connect("more-reviews-clicked", self._on_more_reviews_clicked)
         vb.pack_start(self.reviews, False)
 
         self.show_all()
@@ -1106,8 +1129,42 @@ class AppDetailsViewGtk(gtk.Viewport, AppDetailsViewBase):
         hb.connect('size-allocate', self._header_on_allocate, hb.get_spacing())
         vb.connect('expose-event', self._on_expose, alignment)
         vb.connect('size-allocate', self._on_allocate)
+
+        vb.connect('key-press-event', self._on_key_press)
         return
 
+    def _on_key_press(self, widget, event):
+        kv = event.keyval
+
+        # key values we want to respond to
+        uppers = (gtk.keysyms.KP_Up, gtk.keysyms.Up)
+        downers = (gtk.keysyms.KP_Down, gtk.keysyms.Down)
+
+        if kv in uppers or kv in downers:
+            # get the ScrolledWindow adjustments and tweak them appropriately
+            if not self.parent: return False
+
+            # the ScrolledWindow vertical-adjustment
+            v_adj = self.parent.get_vadjustment()
+
+            # scroll up
+            if kv in uppers:
+                v = max(v_adj.value - v_adj.step_increment,
+                        v_adj.lower)
+
+            # scroll down 
+            elif kv in downers:
+                v = min(v_adj.value + v_adj.step_increment,
+                        v_adj.upper - v_adj.page_size)
+
+            # set our new value
+            v_adj.set_value(v)
+
+            # do not share the event with other widgets
+            return True
+        # share the event with other widgets
+        return False
+
     def _on_review_new(self, button):
         self._review_write_new()
 
@@ -1461,6 +1518,8 @@ class AppDetailsViewGtk(gtk.Viewport, AppDetailsViewBase):
               result and
               result.pkgname):
             self.pkg_statusbar.configure(self.app_details, PKG_STATE_INSTALLED)
+            # reset the reviews UI now that we have installed the package
+            self.reviews.configure_reviews_ui()
         # normal states
         elif state == PKG_STATE_REMOVING:
             self.pkg_statusbar.configure(self.app_details, PKG_STATE_UNINSTALLED)
@@ -1476,6 +1535,8 @@ class AppDetailsViewGtk(gtk.Viewport, AppDetailsViewBase):
         # cancellation of dependency dialog
         elif state == PKG_STATE_INSTALLED:
             self.pkg_statusbar.configure(self.app_details, PKG_STATE_INSTALLED)
+            # reset the reviews UI now that we have installed the package
+            self.reviews.configure_reviews_ui()
         elif state == PKG_STATE_UNINSTALLED:
             self.pkg_statusbar.configure(self.app_details, PKG_STATE_UNINSTALLED)
         self.adjustment_value = None
