@@ -27,11 +27,6 @@ import logging
 import apt_pkg
 apt_pkg.init_config()
 
-try:
-    from debian import deb822
-except ImportError:
-    from debian_bundle import deb822
-
 import os.path
 import datetime
 
@@ -39,6 +34,7 @@ from gettext import gettext as _
 
 from softwarecenter.enums import *
 from softwarecenter.view.widgets.searchentry import SearchEntry
+from softwarecenter.view.widgets.spinner import SpinnerView
 from softwarecenter.apt.aptcache import AptCache
 from softwarecenter.db.database import StoreDatabase
 from softwarecenter.view.basepane import BasePane
@@ -50,6 +46,9 @@ class HistoryPane(gtk.VBox, BasePane):
                               gobject.TYPE_NONE, 
                               (int, ),
                              ),
+        "history-pane-created" : (gobject.SIGNAL_RUN_FIRST,
+                                  gobject.TYPE_NONE,
+                                  ()),
     }
 
     (COL_WHEN, COL_ACTION, COL_PKG) = range(3)
@@ -59,8 +58,12 @@ class HistoryPane(gtk.VBox, BasePane):
 
     ICON_SIZE = 24
     PADDING = 6
+    
+    # pages for the spinner notebook
+    (PAGE_HISTORY_VIEW,
+     PAGE_SPINNER) = range(2)
 
-    def __init__(self, cache, history, db, distro, icons, datadir):
+    def __init__(self, cache, db, distro, icons, datadir):
         gtk.VBox.__init__(self)
         self.cache = cache
         self.db = db
@@ -118,15 +121,27 @@ class HistoryPane(gtk.VBox, BasePane):
         removals_action.set_group(all_action)
         removals_button = removals_action.create_tool_item()
         self.toolbar.insert(removals_button, 3)
+        
+        self._actions_list = all_action.get_group()
+        self._set_actions_sensitive(False)
 
         self.view = gtk.TreeView()
         self.view.show()
-        self.scrolled_view = gtk.ScrolledWindow()
-        self.scrolled_view.set_policy(gtk.POLICY_AUTOMATIC,
+        self.history_view = gtk.ScrolledWindow()
+        self.history_view.set_policy(gtk.POLICY_AUTOMATIC,
                                       gtk.POLICY_AUTOMATIC)
-        self.scrolled_view.show()
-        self.scrolled_view.add(self.view)
-        self.pack_start(self.scrolled_view)
+        self.history_view.show()
+        self.history_view.add(self.view)
+        
+        # make a spinner to display while history is loading
+        self.spinner_view = SpinnerView(_('Loading history'))
+        self.spinner_notebook = gtk.Notebook()
+        self.spinner_notebook.set_show_tabs(False)
+        self.spinner_notebook.set_show_border(False)
+        self.spinner_notebook.append_page(self.history_view)
+        self.spinner_notebook.append_page(self.spinner_view)
+        
+        self.pack_start(self.spinner_notebook)
 
         self.store = gtk.TreeStore(*self.COL_TYPES)
         self.visible_changes = 0
@@ -137,10 +152,9 @@ class HistoryPane(gtk.VBox, BasePane):
         self.filename = apt_pkg.config.find_file("Dir::Log::History")
         self.last = None
         
-        self.history = history
-        self.parse_history()
-        self.history.set_on_update(self.parse_history)
-        
+        # to save (a lot of) time at startup we load history later, only when
+        # it is selected to be viewed
+        self.history = None
 
         self.column = gtk.TreeViewColumn(_('Date'))
         self.view.append_column(self.column)
@@ -150,6 +164,27 @@ class HistoryPane(gtk.VBox, BasePane):
         self.cell_text = gtk.CellRendererText()
         self.column.pack_start(self.cell_text)
         self.column.set_cell_data_func(self.cell_text, self.render_cell_text)
+        
+        # busy cursor
+        self.busy_cursor = gtk.gdk.Cursor(gtk.gdk.WATCH)
+        
+    def init_view(self):
+        if self.history == None:
+            # if the history is not yet initialized we have to load and parse it
+            # show a spinner while we do that
+            self.window.set_cursor(self.busy_cursor)
+            self.spinner_view.start()
+            self.spinner_notebook.set_current_page(self.PAGE_SPINNER)
+            self.load_and_parse_history()
+            self.spinner_notebook.set_current_page(self.PAGE_HISTORY_VIEW)
+            self.spinner_view.stop()
+            self._set_actions_sensitive(True)
+            self.window.set_cursor(None)
+            self.emit("history-pane-created")
+            
+    def _set_actions_sensitive(self, sensitive):
+        for action in self._actions_list:
+            action.set_sensitive(sensitive)
 
     def _reset_icon_cache(self, theme=None):
         self._app_icon_cache.clear()
@@ -158,6 +193,16 @@ class HistoryPane(gtk.VBox, BasePane):
         except glib.GError:
             missing = None
         self._app_icon_cache[MISSING_APP_ICON] = missing
+        
+    def load_and_parse_history(self):
+        from softwarecenter.apt.apthistory import get_apt_history
+        self.history = get_apt_history()
+        # FIXME: a signal from AptHistory is nicer
+        while not self.history.history_ready:
+            while gtk.events_pending():
+                gtk.main_iteration()
+        self.parse_history()
+        self.history.set_on_update(self.parse_history)
 
     def parse_history(self):
         date = None
@@ -171,6 +216,8 @@ class HistoryPane(gtk.VBox, BasePane):
             return
         new_last = self.history.transactions[0].start_date
         for trans in self.history.transactions:
+            while gtk.events_pending():
+                gtk.main_iteration()
             when = trans.start_date
             if self.last is not None and when <= self.last:
                 break
@@ -183,8 +230,7 @@ class HistoryPane(gtk.VBox, BasePane):
                        self.UPGRADED: trans.upgrade,
                       }
             for action, pkgs in actions.iteritems():
-                pkgnames = [p.split(':')[0].split(' (')[0] for p in pkgs]
-                for pkgname in pkgnames:
+                for pkgname in pkgs:
                     row = (when, action, pkgname)
                     last_row = self.store.insert_after(day, last_row, row)
         self.last = new_last
@@ -202,7 +248,7 @@ class HistoryPane(gtk.VBox, BasePane):
 
     def update_view(self):
         self.store_filter.refilter()
-
+        self.view.collapse_all()
         # Expand all the matching rows
         if self.searchentry.get_text():
             self.view.expand_all()
@@ -213,6 +259,13 @@ class HistoryPane(gtk.VBox, BasePane):
         while day is not None:
             self.visible_changes += self.store_filter.iter_n_children(day)
             day = self.store_filter.iter_next(day)
+            
+        # Expand the most recent day
+        day = self.store.get_iter_first()
+        if day is not None:
+	        path = self.store.get_path(day)
+	        self.view.expand_row(path, False)
+	        self.view.scroll_to_cell(path)
 
         self.emit('app-list-changed', self.visible_changes)
 
@@ -264,12 +317,14 @@ class HistoryPane(gtk.VBox, BasePane):
         if isinstance(when, datetime.datetime):
             action = store.get_value(iter, self.COL_ACTION)
             pkg = store.get_value(iter, self.COL_PKG)
+	    # Translators : time displayed in history, display hours (0-12), minutes and AM/PM. %H should be used instead of %I to display hours 0-24
+            subs = {'pkgname': pkg, 'time': when.time().strftime(_('%I:%M %p'))}
             if action == self.INSTALLED:
-                text = _('%s installed %s') % (pkg, when.time().strftime('%X'))
+                text = _('%(pkgname)s installed %(time)s') % subs
             elif action == self.REMOVED:
-                text = _('%s removed %s') % (pkg, when.time().strftime('%X'))
+                text = _('%(pkgname)s removed %(time)s') % subs
             elif action == self.UPGRADED:
-                text = _('%s upgraded %s') % (pkg, when.time().strftime('%X'))
+                text = _('%(pkgname)s updated %(time)s') % subs
         elif isinstance(when, datetime.date):
             today = datetime.date.today()
             monday = today - datetime.timedelta(days=today.weekday())
@@ -304,6 +359,7 @@ if __name__ == '__main__':
     window.set_size_request(600, 500)
     window.set_position(gtk.WIN_POS_CENTER)
     window.show_all()
+    widget.init_view()
     window.connect('destroy', gtk.main_quit)
 
     gtk.main()
