@@ -130,7 +130,10 @@ class NavigationItem(object):
         self.apps_category = available_pane.apps_category
         self.apps_subcategory = available_pane.apps_subcategory
         self.apps_search_term = available_pane.apps_search_term
-        self.current_app = available_pane.get_current_app()
+        if available_pane.is_applist_view_showing():
+            self.current_app = None
+        else:
+            self.current_app = available_pane.get_current_app()
         self.parts = self.available_pane.navigation_bar.get_parts()
 
     def navigate_to(self):
@@ -145,7 +148,7 @@ class NavigationItem(object):
         available_pane.apps_search_term = self.apps_search_term
         available_pane.searchentry.set_text(self.apps_search_term)
         available_pane.searchentry.set_position(-1)
-        available_pane.app_details.show_app(self.current_app)
+        available_pane.app_details_view.show_app(self.current_app)
 
         nav_bar = self.available_pane.navigation_bar
         nav_bar.remove_all(do_callback=False, animate=False)
