@@ -31,6 +31,7 @@ import aptdaemon.client
 from aptdaemon.enums import *
 
 from softwarecenter.enums import *
+from softwarecenter.utils import get_icon_from_theme
 from softwarecenter.backend import get_install_backend
 from softwarecenter.backend.transactionswatcher import TransactionsWatcher
 from softwarecenter.view.basepane import BasePane
@@ -99,21 +100,12 @@ class PendingStore(gtk.ListStore, TransactionsWatcher):
         # add pending purchases as pseudo transactions
         for pkgname in self.backend.pending_purchases:
             iconname = self.backend.pending_purchases[pkgname].iconname
-            icon = self._get_icon_from_iconname(iconname)
+            icon = get_icon_from_theme(self.icons, iconname=iconname, iconsize=self.ICON_SIZE)
             appname = self.backend.pending_purchases[pkgname].appname
             status_text = self._render_status_text(
                 appname or pkgname, _(u'Installing purchase\u2026'))
             self.append([pkgname, icon, pkgname, status_text, 0, 1, None])
 
-    def _get_icon_from_iconname(self, iconname=None):
-        if not iconname:
-            iconname = MISSING_APP_ICON
-        try:
-            icon = self.icons.load_icon(iconname, self.ICON_SIZE, 0)
-        except Exception:
-            icon = self.icons.load_icon(MISSING_APP_ICON, self.ICON_SIZE, 0)
-        return icon
-
     def _pulse_purchase_helper(self):
         for item in self:
             if item[self.COL_PULSE] > 0:
@@ -125,13 +117,16 @@ class PendingStore(gtk.ListStore, TransactionsWatcher):
         store.
         """
         logging.debug("_append_transaction %s (%s)" % (trans.tid, trans))
+        self._signals.append(
+            trans.connect(
+                "progress-details-changed", self._on_progress_details_changed))
         self._signals.append(
             trans.connect("progress-changed", self._on_progress_changed))
         self._signals.append(
             trans.connect("status-changed", self._on_status_changed))
         self._signals.append(
-            trans.connect("cancellable-changed",
-                          self._on_cancellable_changed))
+            trans.connect(
+                "cancellable-changed",self._on_cancellable_changed))
 
         if "sc_appname" in trans.meta_data:
             appname = trans.meta_data["sc_appname"]
@@ -145,9 +140,9 @@ class PendingStore(gtk.ListStore, TransactionsWatcher):
         try:
             iconname = trans.meta_data["sc_iconname"]
         except KeyError:
-            icon = self._get_icon_from_iconname()
+            icon = get_icon_from_theme(self.icons, iconsize=self.ICON_SIZE)
         else:
-            icon = self._get_icon_from_iconname(iconname)
+            icon = get_icon_from_theme(self.icons, iconname=iconname, iconsize=self.ICON_SIZE)
         if trans.status == STATUS_WAITING_LOCK:
             status = trans.status_details
         else:
@@ -175,19 +170,27 @@ class PendingStore(gtk.ListStore, TransactionsWatcher):
             if row[self.COL_TID] == trans.tid:
                 row[self.COL_NAME] = get_role_localised_present_from_enum(role)
 
-    def _on_progress_changed(self, trans, progress):
-        #print "_on_progress_changed: ", trans, progress
+    def _on_progress_details_changed(self, trans, current_items, total_items,
+                                     current_bytes, total_bytes, current_cps,
+                                     eta):
+        #print "_on_progress_details_changed: ", trans, progress
         for row in self:
             if row[self.COL_TID] == trans.tid:
-                row[self.COL_PROGRESS] = progress
                 if trans.status == STATUS_DOWNLOADING:
                     name = row[self.COL_NAME]
-                    current_bytes = apt_pkg.size_to_str(trans.progress_details[2])
-                    total_bytes = apt_pkg.size_to_str(trans.progress_details[3])
+                    current_bytes_str = apt_pkg.size_to_str(current_bytes)
+                    total_bytes_str = apt_pkg.size_to_str(total_bytes)
                     status = _("Downloaded %sB of %sB") % \
-                             (current_bytes, total_bytes)
+                             (current_bytes_str, total_bytes_str)
                     row[self.COL_STATUS] = self._render_status_text(name, status)
 
+    def _on_progress_changed(self, trans, progress):
+        # print "_on_progress_changed: ", trans, progress
+        for row in self:
+            if row[self.COL_TID] == trans.tid:
+                if progress:
+                    row[self.COL_PROGRESS] = progress
+
     def _on_status_changed(self, trans, status):
         #print "_on_progress_changed: ", trans, status
         for row in self:
@@ -207,30 +210,33 @@ class PendingStore(gtk.ListStore, TransactionsWatcher):
         return "%s\n<small>%s</small>" % (name, status)
 
 
-class PendingView(gtk.TreeView, BasePane):
+class PendingView(gtk.ScrolledWindow, BasePane):
     
     CANCEL_XPAD = 6
     CANCEL_YPAD = 6
 
     def __init__(self, icons):
-        gtk.TreeView.__init__(self)
+        gtk.ScrolledWindow.__init__(self)
         BasePane.__init__(self)
+        self.tv = gtk.TreeView()
         # customization
-        self.set_headers_visible(False)
-        self.connect("button-press-event", self._on_button_pressed)
+        self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        self.add(self.tv)
+        self.tv.set_headers_visible(False)
+        self.tv.connect("button-press-event", self._on_button_pressed)
         # icon
         self.icons = icons
         tp = gtk.CellRendererPixbuf()
         tp.set_property("xpad", self.CANCEL_XPAD)
         tp.set_property("ypad", self.CANCEL_YPAD)
         column = gtk.TreeViewColumn("Icon", tp, pixbuf=PendingStore.COL_ICON)
-        self.append_column(column)
+        self.tv.append_column(column)
         # name
         tr = gtk.CellRendererText()
         column = gtk.TreeViewColumn("Name", tr, markup=PendingStore.COL_STATUS)
         column.set_min_width(200)
         column.set_expand(True)
-        self.append_column(column)
+        self.tv.append_column(column)
         # progress
         tp = gtk.CellRendererProgress()
         tp.set_property("xpad", self.CANCEL_XPAD)
@@ -240,25 +246,25 @@ class PendingView(gtk.TreeView, BasePane):
                                     value=PendingStore.COL_PROGRESS,
                                     pulse=PendingStore.COL_PULSE)
         column.set_min_width(200)
-        self.append_column(column)
+        self.tv.append_column(column)
         # cancel icon
         tpix = gtk.CellRendererPixbuf()
         column = gtk.TreeViewColumn("Cancel", tpix, 
                                     stock_id=PendingStore.COL_CANCEL)
-        self.append_column(column)
+        self.tv.append_column(column)
         # fake columns that eats the extra space at the end
         tt = gtk.CellRendererText()
         column = gtk.TreeViewColumn("Cancel", tt)
-        self.append_column(column)
+        self.tv.append_column(column)
         # add it
         store = PendingStore(icons)
-        self.set_model(store)
+        self.tv.set_model(store)
     def _on_button_pressed(self, widget, event):
         """button press handler to capture clicks on the cancel button"""
         #print "_on_clicked: ", event
         if event == None or event.button != 1:
             return
-        res = self.get_path_at_pos(int(event.x), int(event.y))
+        res = self.tv.get_path_at_pos(int(event.x), int(event.y))
         if not res:
             return
         (path, column, wx, wy) = res
@@ -269,7 +275,7 @@ class PendingView(gtk.TreeView, BasePane):
         if column.get_title() != "Cancel":
             return
         # not cancelable (no icon)
-        model = self.get_model()
+        model = self.tv.get_model()
         if model[path][PendingStore.COL_CANCEL] == "":
             return 
         # get tid
