@@ -78,14 +78,7 @@ class SoftwareSection(object):
         cr.rectangle(0,0,a.width, 150)
         cr.fill()
 
-        # there is a race here because we create e.g. the installed-page
-        # delayed. if its not created yet, we just do not show a image
-        # until its available
-        if self._image_id in self.MASK_SURFACE_CACHE:
-            s = self.MASK_SURFACE_CACHE[self._image_id]
-        else:
-            s = cairo.ImageSurface(0, 64, 64)
-
+        s = self.MASK_SURFACE_CACHE[self._image_id]
         if widget.get_direction() != gtk.TEXT_DIR_RTL:
             cr.set_source_surface(s, a.x+a.width-s.get_width(), 0)
         else:
@@ -425,7 +418,6 @@ class SoftwarePane(gtk.VBox, BasePane):
         launcher
         """
         # TODO: handle local deb install case
-        # TODO: handle apps in PPAs and in extras.ubuntu.com
         # TODO: implement the list view case (once it is specified)
         # only show the panel if unity is running and this is a package install
         #
@@ -727,6 +719,10 @@ class SoftwarePane(gtk.VBox, BasePane):
         # set model
         self.app_view.set_model(new_model)
         self.app_view.get_model().active = True
+        
+        # connect the refresh signal from the model
+        new_model.connect(
+            "needs-refresh", lambda helper, pkgname: self.app_view.queue_draw())
 
         self.hide_appview_spinner()
         # we can not use "new_model" here, because set_model may actually
