@@ -25,6 +25,9 @@ import cairo
 import pango
 import gobject
 import pangocairo
+from math import pi, cos, sin
+
+from math import sin, cos
 
 from mkit_themes import Color, ColorArray, ThemeRegistry
 
@@ -122,8 +125,8 @@ def not_overlapping(widget_area, expose_area):
 #######################
 
 # pi constants
-PI =            3.1415926535897931
-PI_OVER_180 =   0.017453292519943295
+PI =            pi
+PI_OVER_180 =   pi / 180
 
 # shapes constants
 SHAPE_RECTANGLE =   0
@@ -153,13 +156,7 @@ SPACING_SMALL       = max(1, int(0.333*EM+0.5))
 
 # recommended corner radius
 CORNER_RADIUS = 0
-
-# use the link color as the clicked color for labels
-_scheme = get_gtk_color_scheme_dict()
-if _scheme and _scheme.has_key('link_color'):
-    LINK_ACTIVE_COLOR = _scheme['link_color']
-else:
-    LINK_ACTIVE_COLOR = '#FF0000'   # red
+LINK_ACTIVE_COLOR = '#FF0000'   # red
 
 
 
@@ -237,6 +234,8 @@ def get_mkit_theme():
 
     return CACHED_THEME
 
+def radian(deg):
+    return PI_OVER_180 * deg
 
 
 
@@ -244,6 +243,7 @@ def get_mkit_theme():
 ### HANDY CLASSES ###
 #####################
 
+
 class Shape:
 
     """ Base class for a Shape implementation.
@@ -470,6 +470,37 @@ class ShapeCircle(Shape):
         return
 
 
+class ShapeStar(Shape):
+
+    def __init__(self, points, indent=0.61, direction=gtk.TEXT_DIR_LTR):
+        self.coords = self._calc_coords(points, 1-indent)
+
+    def _calc_coords(self, points, indent):
+        coords = []
+        step = radian(180.0/points)
+
+        for i in range(2*points):
+            if i%2:
+                x = (sin(step*i)+1)*0.5
+                y = (cos(step*i)+1)*0.5
+            else:
+                x = (sin(step*i)*indent+1)*0.5
+                y = (cos(step*i)*indent+1)*0.5
+
+            coords.append((x,y))
+        return coords
+
+    def layout(self, cr, x, y, w, h):
+        points = map(lambda (sx,sy): (sx*w+x,sy*h+y), self.coords)
+        cr.move_to(*points[0])
+
+        for p in points[1:]:
+            cr.line_to(*p)
+
+        cr.close_path()
+        return
+
+
 class Style:
 
     def __init__(self):
@@ -697,200 +728,272 @@ class Style:
         return
 
 
-class FramedSection(gtk.VBox):
+class LayoutView2(gtk.HBox):
 
-    def __init__(self, label_markup=None, xpadding=SPACING_MED):
-        gtk.VBox.__init__(self)
-        self.set_redraw_on_allocate(False)
-
-        self.header_alignment = gtk.Alignment(xscale=1.0, yscale=1.0)
-        self.header = gtk.HBox()
-        self.header_alignment.add(self.header)
-        self.header_alignment.set_padding(SPACING_SMALL,
-                                          SPACING_SMALL,
-                                          xpadding,
-                                          xpadding)
-
-        self.body_alignment = gtk.Alignment(xscale=1.0, yscale=1.0)
-        self.body = gtk.VBox()
-        self.body_alignment.add(self.body)
-        self.body_alignment.set_padding(SPACING_MED, 0, 0, 0)
-
-        self.footer_alignment = gtk.Alignment(xscale=1.0, yscale=1.0)
-        self.footer = gtk.HBox()
-        self.footer_alignment.add(self.footer)
-        self.footer_alignment.set_padding(0, 0,
-                                          xpadding,
-                                          xpadding)
-
-        self.body.set_spacing(SPACING_MED)
-        self.footer.set_size_request(-1, 2*EM)
-
-        self.pack_start(self.header_alignment, False)
-        self.pack_start(self.body_alignment)
-        self.pack_start(self.footer_alignment, False)
-
-        self.image = gtk.Image()
-        self.label = EtchedLabel()
-        # Make sure the user can select and copy the title/summary
-        #self.label.set_selectable(True)
-        self.header.pack_start(self.label, False)
-        if label_markup:
-            self.set_label(label_markup)
-        return
-
-    def set_icon_from_name(self, icon_name, icon_size=gtk.ICON_SIZE_MENU):
-        self.image.set_from_icon_name(icon_name, icon_size)
-
-        if not self.image.parent:
-            self.header.pack_start(self.image, False)
-            self.header.reorder_child(self.image, 0)
-            self.image.show()
-        return
-
-    def set_icon_from_pixbuf(self, pixbuf):
-        self.image.set_from_pixbuf(pixbuf)
-
-        if not self.image.parent:
-            self.header.pack_start(self.image, False)
-            self.header.reorder_child(self.image, 0)
-            self.image.show()
-        return
-
-    def set_label(self, label='', markup=None):
-        if markup:
-            self.label.set_markup(markup)
-        else:
-            self.label.set_markup('<b>%s</b>' % label)
-
-        # atk stuff
-        acc = self.get_accessible()
-        acc.set_name(self.label.get_text())
-        acc.set_role(atk.ROLE_SECTION)
-        return
-
-    def set_xpadding(self, xpadding):
-        self.header_alignment.set_padding(0, 0, xpadding, xpadding)
-        self.body_alignment.set_padding(0, 0, xpadding, xpadding)
-        self.footer_alignment.set_padding(0, 0, xpadding, xpadding)
-        return
-
-    def draw(self, cr, a, expose_area, draw_border=True):
-#        if not_overlapping(a, expose_area): return
-
-#        cr.save()
-#        cr.translate(0.5, 0.5)
-#        cr.rectangle(a)
-#        cr.clip_preserve()
-
-        #a = self.header_alignment.allocation
-
-        # fill section white
-        #cr.rectangle(a)
-        #cr.set_source_rgb(*floats_from_gdkcolor_wit(self.style.base[self.state]))
-        #cr.fill()
-        #cr.save()
-#        cr.set_line_width(1)
-#        cr.set_source_rgb(*floats_from_gdkcolor(self.style.dark[self.state]))
-#        cr.stroke()
-        #cr.restore()
-
-#        cr.restore()
-        return
-
-
-class LayoutView(FramedSection):
-
-    def __init__(self):
-        FramedSection.__init__(self)
-        self.set_redraw_on_allocate(False)
-
-        self.column_hbox = gtk.HBox(spacing=SPACING_SMALL)
-        self.column_hbox.set_homogeneous(True)
-        self.body.pack_start(self.column_hbox)
+    def __init__(self, xspacing=4, yspacing=6):
+        gtk.HBox.__init__(self, spacing=xspacing)
+        self.set_homogeneous(True)
 
+        self.min_col_width = 128
         self.n_columns = 0
-        self.widget_list = []
-        self.theme = get_mkit_theme()
+        self.yspacing = yspacing
+
+        self._allocation = None
+        self._non_col_children = []
+
+        self.connect('size-allocate', self._on_allocate, yspacing)
+        #~ self.connect('expose-event', self._on_expose_debug)
         return
 
-    def append(self, widget):
-        self.widget_list.append(widget)
-        return
+    def _on_allocate(self, widget, allocation, yspacing):
+        if self._allocation == allocation: return True
+        self._allocation = allocation
+        w = allocation.width
+        self.layout(w, yspacing, force=False)
+        return True
 
-    def set_width(self, width):
-        self.body.set_size_request(width, -1)
+    def _on_expose_debug(self, widget, event):
+        cr = widget.window.cairo_create()
 
-        # find widest button/widget
-        widgets = self.widget_list
-        widest_w = 0
-        for btn in widgets:
-            widest_w = max(widest_w, btn.calc_width())
+        for i, child in enumerate(self.get_children()):
+            a = child.allocation
+            cr.rectangle(a)
 
-        # determine number of columns to display
-#        width -= 100    # fudge number
-        n_columns = width / widest_w
-        n_columns = (width - n_columns*self.column_hbox.get_spacing()) / widest_w
-
-        if n_columns > len(widgets):
-            n_columns = len(widgets)
-        if n_columns <= 0:
-            n_columns = 1
-
-        if n_columns == self.n_columns: return
-        self.clear_columns()
-
-        # pack columns into widget
-        for i in range(n_columns):
-            self.column_hbox.pack_start(gtk.VBox(spacing=SPACING_LARGE))
-
-        # pack buttons into appropriate columns
-        i = 0
-        columns = self.column_hbox.get_children()
-        for btn in widgets:
-            columns[i].pack_start(btn, False)
-            if i < n_columns-1:
-                i += 1
+            if i%2:
+                cr.set_dash((2,2))
+                cr.set_source_rgb(1,0,0)
             else:
-                i = 0
+                cr.set_dash((2,2), 2)
+                cr.set_source_rgb(0,1,0)
+
+            cr.stroke()
+
+        del cr
+        return
+
+    def add(self, child):
+        self._non_col_children.append(child)
+        return
+
+    def clear(self):
+        for w in self._non_col_children:
+            w.destroy()
+
+        self._non_col_children = []
+        return
+
+    def set_widgets(self, widgets):
+        self.clear()
+        for w in widgets:
+            self._non_col_children.append(w)
+        
+        gobject.idle_add(self.layout, self.allocation.width, self.yspacing)
+        return
+
+    def layout(self, width, yspacing, force=True):
+        n_cols = max(1, width / (self.min_col_width + self.get_spacing()))
+        n_cols = min(len(self._non_col_children), n_cols)
+
+        if self.n_columns == n_cols and not force:
+            return True
+
+        for i, col in enumerate(self.get_children()):
+            for child in col.get_children():
+                col.remove(child)
+
+            if i >= n_cols:
+                self.remove(col)
+                col.destroy()
+
+        if n_cols > self.n_columns:
+            for i in range(self.n_columns, n_cols):
+                col = gtk.VBox(spacing=yspacing)
+                self.pack_start(col)
+
+        cols = self.get_children()
+        for i, child in enumerate(self._non_col_children):
+            cols[i%n_cols].pack_start(child, False)
 
         self.show_all()
-        self.n_columns = n_columns
+        self.n_columns = n_cols
         return
 
-    def clear_all(self):
-        # destroy all columns and column-children
-        self.widget_list = []
-        self.n_columns = 0
-        for col in self.column_hbox.get_children():
-            for child in col.get_children():
-                child.destroy()
-            col.destroy()
+
+class Button(gtk.EventBox):
+
+    __gsignals__ = {
+        "clicked" : (gobject.SIGNAL_RUN_LAST,
+                     gobject.TYPE_NONE, 
+                     (),)
+        }
+
+    def __init__(self):
+        gtk.EventBox.__init__(self)
+        self.set_visible_window(False)
+
+        self.set_flags(gtk.CAN_FOCUS)
+        self.set_events(gtk.gdk.BUTTON_PRESS_MASK|
+                        gtk.gdk.BUTTON_RELEASE_MASK|
+                        gtk.gdk.KEY_RELEASE_MASK|
+                        gtk.gdk.KEY_PRESS_MASK|
+                        gtk.gdk.ENTER_NOTIFY_MASK|
+                        gtk.gdk.LEAVE_NOTIFY_MASK)
+
+        self._button_press_origin = None
+        self._cursor = gtk.gdk.Cursor(cursor_type=gtk.gdk.HAND2)
+
+        self.connect("button-press-event", self._on_button_press)
+        self.connect("button-release-event", self._on_button_release)
+        self.connect('enter-notify-event', self._on_enter)
+        self.connect('leave-notify-event', self._on_leave)
+        self.connect('key-press-event', self._on_key_press)
         return
 
-    def clear_columns(self):
-        # remove columns, but do not destroy column-children
-        for col in self.column_hbox.get_children():
-            for btn in col.get_children():
-                col.remove(btn)
-            col.destroy()
+    def _on_button_press(self, btn, event):
+        if event.button != 1:
+            self.set_state(gtk.STATE_NORMAL)
+            self.window.set_cursor(None)
+            if hasattr(self, 'label_list'):
+                for v in self.label_list:
+                    l = getattr(self, v)
+                    self._label_colorise_normal(l)
+            return
+        self._button_press_origin = btn
+        self.set_state(gtk.STATE_ACTIVE)
+
+        if hasattr(self, 'label_list'):
+            for v in self.label_list:
+                l = getattr(self, v)
+                self._label_colorise_active(l)
         return
 
-    def draw(self, cr, a, expose_area):
-        if not_overlapping(a, expose_area): return
+    def _on_button_release(self, btn, event):
 
-#        cr.save()
-#        FramedSection.draw(self, cr, a, expose_area)
+        def clicked(w):
+            w.emit('clicked')
 
-        for btn in self.widget_list:
-            a = btn.allocation
-            if a.width == 1 or a.height == 1: break
+        if event.button != 1:
+            self.queue_draw()
+            return
 
-            btn.draw(cr, a, expose_area)
+        region = gtk.gdk.region_rectangle(self.allocation)
+        if not region.point_in(*self.window.get_pointer()[:2]):
+            self._button_press_origin = None
+            self.set_state(gtk.STATE_NORMAL)
+            return
 
-#        cr.restore()
+        self._button_press_origin = None
+        self.set_state(gtk.STATE_PRELIGHT)
+
+        if hasattr(self, 'label_list'):
+            for v in self.label_list:
+                l = getattr(self, v)
+                self._label_colorise_normal(l)
+
+        gobject.timeout_add(50, clicked, btn)
         return
 
+    def _on_enter(self, btn, event):
+        if self == self._button_press_origin:
+            self.set_state(gtk.STATE_ACTIVE)
+        else:
+            self.set_state(gtk.STATE_PRELIGHT)
+
+        if not event.state:
+            self.window.set_cursor(self._cursor)
+
+        if event.state and event.state == gtk.gdk.BUTTON1_MASK:
+            self.window.set_cursor(self._cursor)
+            if hasattr(self, 'label_list'):
+                for v in self.label_list:
+                    l = getattr(self, v)
+                    self._label_colorise_active(l)
+        return
+
+    def _on_leave(self, btn, event):
+        self.set_state(gtk.STATE_NORMAL)
+        self.window.set_cursor(None)
+
+        if hasattr(self, 'label_list'):
+            for v in self.label_list:
+                l = getattr(self, v)
+                self._label_colorise_normal(l)
+        return
+
+    def _on_key_press(self, btn, event):
+        if (event.keyval != (gtk.gdk.keyval_from_name('Return') or
+             gtk.gdk.keyval_from_name('KP_Enter'))):
+            return
+
+        def clicked(w):
+            w.emit('clicked')
+
+        gobject.timeout_add(50, clicked, btn)
+        return
+
+    def _label_colorise_active(self, label):
+        c = self.style.base[gtk.STATE_SELECTED]
+
+        attr = pango.AttrForeground(c.red,
+                                    c.green,
+                                    c.blue,
+                                    0, -1)
+
+        layout = label.get_layout()
+        attrs = layout.get_attributes()
+
+        if not attrs:
+            attrs = pango.AttrList()
+
+        attrs.change(attr)
+        layout.set_attributes(attrs)
+        return
+
+    def _label_colorise_normal(self, label):
+        if self.state == gtk.STATE_PRELIGHT or \
+            self.has_focus():
+            if hasattr(label, 'is_subtle') and label.is_subtle:
+                c = self.style.dark[self.state]
+            else:
+                c = self.style.text[self.state]
+        else:
+            c = self.style.text[self.state]
+
+        attr = pango.AttrForeground(c.red,
+                                    c.green,
+                                    c.blue,
+                                    0, -1)
+
+        layout = label.get_layout()
+        attrs = layout.get_attributes()
+
+        if not attrs:
+            attrs = pango.AttrList()
+
+        attrs.change(attr)
+        layout.set_attributes(attrs)
+        return
+
+
+class LinkButtonLight(Button):
+
+    def __init__(self):
+        Button.__init__(self)
+        self.label = EtchedLabel()
+        self.add(self.label)
+        self.show_all()
+
+        self.label_list = ('label',)
+        return
+
+    def set_label(self, label):
+        self.label.set_markup('<u>%s</u>' % label)
+        return
+
+    def draw(self, *args):
+        return
+
+
 
 class LinkButton(gtk.EventBox):
 
@@ -907,20 +1010,18 @@ class LinkButton(gtk.EventBox):
         self.set_visible_window(False)
         self.set_redraw_on_allocate(False)
 
-        self.alignment = gtk.Alignment(xalign=0.5)
-        self.add(self.alignment)
-
         self.label = EtchedLabel()
         self.label.set_etching_alpha(0.55)
         self.image = gtk.Image()
 
+        self._max_w = -1
+        self._max_h = -1
+
         self._layout = None
         self._button_press_origin = None    # broken?
         self._cursor = gtk.gdk.Cursor(cursor_type=gtk.gdk.HAND2)
-        self._markup = None
         self._use_underline = False
         self._subdued = False
-        self._xmargin = 3
         self.alpha = 1.0
 
         if markup:
@@ -988,16 +1089,16 @@ class LinkButton(gtk.EventBox):
 
     def _on_enter(self, cat, event):
         if cat == self._button_press_origin:
-            self._colorise_label_active()
             cat.set_state(gtk.STATE_ACTIVE)
         else:
             cat.set_state(gtk.STATE_PRELIGHT)
+        self._colorise_label_normal()
         self.window.set_cursor(self._cursor)
         return
 
     def _on_leave(self, cat, event):
-        self._colorise_label_normal()
         cat.set_state(gtk.STATE_NORMAL)
+        self._colorise_label_normal()
         self.window.set_cursor(None)
         return
 
@@ -1018,8 +1119,8 @@ class LinkButton(gtk.EventBox):
     def _on_button_press(self, cat, event):
         if event.button != 1: return
         self._button_press_origin = cat
-        self._colorise_label_active()
         cat.set_state(gtk.STATE_ACTIVE)
+        self._colorise_label_active()
         return
 
     def _on_button_release(self, cat, event):
@@ -1031,16 +1132,15 @@ class LinkButton(gtk.EventBox):
             self.queue_draw()
             return
 
-        self._colorise_label_normal()
-
         cat_region = gtk.gdk.region_rectangle(cat.allocation)
         if not cat_region.point_in(*self.window.get_pointer()[:2]):
             self._button_press_origin = None
             cat.set_state(gtk.STATE_NORMAL)
-            return
-
-        self._button_press_origin = None
-        cat.set_state(gtk.STATE_PRELIGHT)
+        else:
+            self._button_press_origin = None
+            cat.set_state(gtk.STATE_PRELIGHT)
+        
+        self._colorise_label_normal()
         gobject.timeout_add(50, emit_clicked)
         return
 
@@ -1064,22 +1164,40 @@ class LinkButton(gtk.EventBox):
         return
 
     def _colorise_label_active(self):
-        if self._use_underline:
-            self.label.set_markup('<span color="%s"><u>%s</u></span>' % (LINK_ACTIVE_COLOR, self._markup))
-        else:
-            self.label.set_markup('<span color="%s">%s</span>' % (LINK_ACTIVE_COLOR, self._markup))
+        c = gtk.gdk.color_parse(LINK_ACTIVE_COLOR)
+        attr = pango.AttrForeground(c.red,
+                                    c.green,
+                                    c.blue,
+                                    0, -1)
+
+        layout = self.label.get_layout()
+        attrs = layout.get_attributes()
+        if not attrs:
+            attrs = pango.AttrList()
+
+        attrs.change(attr)
+        layout.set_attributes(attrs)
         return
 
     def _colorise_label_normal(self):
-        if not self._subdued or self.has_focus():
-            col = self.style.text[gtk.STATE_NORMAL].to_string()
+        if not self._subdued or self.state == gtk.STATE_PRELIGHT or \
+            self.has_focus():
+            c = self.style.text[self.state]
         else:
-            col = self.style.dark[gtk.STATE_NORMAL].to_string()
+            c = self.style.dark[self.state]
 
-        if self._use_underline:
-            self.label.set_markup('<span color="%s"><u>%s</u></span>' % (col, self._markup))
-        else:
-            self.label.set_markup('<span color="%s">%s</span>' % (col, self._markup))
+        attr = pango.AttrForeground(c.red,
+                                    c.green,
+                                    c.blue,
+                                    0, -1)
+
+        layout = self.label.get_layout()
+        attrs = layout.get_attributes()
+        if not attrs:
+            attrs = pango.AttrList()
+
+        attrs.change(attr)
+        layout.set_attributes(attrs)
         return
 
     def _cache_image_surface(self, pb):
@@ -1096,10 +1214,14 @@ class LinkButton(gtk.EventBox):
 
     def set_underline(self, use_underline):
         self._use_underline = use_underline
+
         if use_underline:
-            self.label.set_markup('<u>%s</u>' % self.label.get_text())
+            l = self.label.get_label()
+            self.label.set_label('<u>%s</u>' % l)
         else:
-            self.label.set_markup(self.label.get_text())
+            m = self.label.get_markup()
+            m.replace('<u>', '').replace('</u>', '')
+            self.label.set_markup(m)
         return
 
     def set_subdued(self, is_subdued):
@@ -1109,13 +1231,12 @@ class LinkButton(gtk.EventBox):
         return
 
     def set_xmargin(self, xmargin):
-        self._xmargin = xmargin
         return
 
     def set_internal_xalignment(self, xalign):
-        self.alignment.set(xalign,
-                           self.alignment.get_property('yalign'),
-                           0, 0)    # x/y scale
+        #~ self.alignment.set(xalign,
+                           #~ self.alignment.get_property('yalign'),
+                           #~ 0, 0)    # x/y scale
         return
 
     def set_internal_spacing(self, internal_spacing):
@@ -1127,7 +1248,6 @@ class LinkButton(gtk.EventBox):
             self.label.set_markup('<u>%s</u>' % label)
         else:
             self.label.set_markup(label)
-        self._markup = label
         w = self.calc_width()
         self.set_size_request(w, self.get_size_request()[1])
         return
@@ -1155,7 +1275,6 @@ class LinkButton(gtk.EventBox):
 
         if self.has_focus() and focus_draw:
             a = self.label.allocation
-            m = self._xmargin
             x, y, w, h = a.x, a.y, a.width, a.height
             self.style.paint_focus(self.window,
                                    self.state,
@@ -1179,26 +1298,22 @@ class EtchedLabel(gtk.Label):
         return
 
     def _on_expose(self, widget, event):
-        l = self.get_layout()
-        a = widget.allocation
-        pc = pangocairo.CairoContext(widget.window.cairo_create())
+        x, y = widget.get_layout_offsets()
 
+        cr = widget.window.cairo_create()
+        cr.rectangle(event.area)
+        cr.clip()
 
-        x, y = a.x, a.y+1
-        lw, lh = l.get_pixel_extents()[1][2:]
-        ax, ay = self.get_alignment()
-
-        if lw < a.width:
-            x += int((a.width-lw)*ax)
-        if lh < a.height:
-            y += int((a.height-lh)*ay)
-
-        pc.move_to(x, y)
-        pc.layout_path(l)
+        cr.move_to(x, y + 1)
+        cr.layout_path(self.get_layout())
         r,g,b = floats_from_gdkcolor(self.style.light[self.state])
-        pc.set_source_rgba(r,g,b,self.alpha)
-        pc.fill()
-        del pc
+        cr.set_source_rgba(r,g,b,self.alpha)
+        cr.fill()
+
+        del cr
+        return
+
+    def set_max_line_count(self, *args):
         return
 
 
@@ -1208,12 +1323,12 @@ class HLinkButton(LinkButton):
         LinkButton.__init__(self, markup, icon_name, icon_size)
 
         self.box = gtk.HBox()
-        self.alignment.add(self.box)
+        self.add(self.box)
 
         if not self.image.get_storage_type() == gtk.IMAGE_EMPTY:
             self.box.pack_start(self.image, False)
         if self.label.get_text():
-            a = gtk.Alignment(1.0, 0.5)
+            a = gtk.Alignment(1.0, 0.5, yscale=1.0)
             a.add(self.label)
             self.box.pack_start(a, False)
 
@@ -1234,60 +1349,146 @@ class HLinkButton(LinkButton):
             w += self.image.allocation.width
             spacing = self.box.get_spacing()
 
-        w += 2*self.get_border_width() + spacing + 2*self._xmargin
+        w += 2*self.get_border_width() + spacing
         return w
 
 
 class VLinkButton(LinkButton):
 
-    MAX_WIDTH  = None
-    MAX_HEIGHT = None
-
     def __init__(self, markup=None, icon_name=None, icon_size=20, icons=None):
         LinkButton.__init__(self, markup, icon_name, icon_size)
 
-        self._xmargin = 6
-
-        self.box = gtk.VBox()
-        self.alignment.add(self.box)
+        self.box = gtk.VBox(spacing=SPACING_SMALL)
+        self.add(self.box)
 
         if not self.image.get_storage_type() == gtk.IMAGE_EMPTY:
             self.box.pack_start(self.image, False)
         if self.label.get_text():
-            self.label_alignment = gtk.Alignment(0.5, 0, xscale=1.0)
-            self.box.pack_end(self.label_alignment)
-            self.label_alignment.add(self.label)
+            self.box.pack_start(self.label)
+            self.label.set_max_line_count(2)
+            self.label.set_alignment(0.5, 0)
+            self.label.set_justify(gtk.JUSTIFY_CENTER)
+            self.box.reorder_child(self.label, -1)
 
         self.set_border_width(BORDER_WIDTH_SMALL)
         self.show_all()
+        #self.connect('expose-event', self._DEBUG_on_expose)
         return
 
+    def set_max_width(self, w):
+        self._max_w = w
+
     def calc_width(self):
         w = 1
         iw = 0
+
         if self.label:
             pc = self.get_pango_context()
             layout = pango.Layout(pc)
             layout.set_markup(self.label.get_label())
             lw = layout.get_pixel_extents()[1][2]   # label width
+
         if self.image and self.image.get_property('visible'):
             iw = self.image.allocation.width        # image width
 
-        w += max(lw, iw)
-        w += 2*self.get_border_width() + 2*self._xmargin
-        
-        if self.MAX_WIDTH and w > self.MAX_WIDTH:
-            if self.label:
-                self.label.set_line_wrap(True)
-                self.label.set_justify(gtk.JUSTIFY_CENTER)
-                self.connect('size-allocate', self._on_allocate_set_label_width)
-            return self.MAX_WIDTH
+        w = max(lw, iw) + 2*self.get_border_width()
+
+        if self._max_w > 0 and w >= self._max_w:
+            w = self._max_w
         return w
-    
-    def _on_allocate_set_label_width(self, widget, allocation):
-        if not self.label: return
-        layout = self.label.get_layout()
-        layout.set_width(pango.SCALE*self.MAX_WIDTH)
-        w, h = layout.get_pixel_extents()[1][2:]
-        self.label.set_size_request(w, h)
+
+    def _DEBUG_on_expose(self, widget, event):
+        # handy for debugging layouts
+        # draw bounding boxes
+        cr = widget.window.cairo_create()
+        cr.set_source_rgba(0, 0, 1, 0.5)
+        cr.rectangle(widget.allocation)
+        cr.stroke()
+
+        cr.translate(0.5,0.5)
+        cr.set_line_width(1)
+
+        cr.set_source_rgb(1, 0, 1)
+        cr.rectangle(self.image.allocation)
+        cr.stroke()
+
+        cr.set_source_rgb(1, 0, 0)
+        cr.rectangle(self.label.allocation)
+        cr.stroke()
         return
+        cr.set_source_rgb(0, 1, 0)
+        x,y,w,h = self.label.get_layout().get_pixel_extents()[1]
+        x += self.label.allocation.x
+        y += self.label.allocation.y
+        cr.rectangle(x,y,w,h)
+        cr.stroke()
+        del cr
+
+
+class BubbleLabel(gtk.Label):
+
+    def __init__(self):
+        gtk.Label.__init__(self)
+        self.set_alignment(0,0)
+        self.set_padding(4, 0)
+        self.shape = ShapeRoundedRectangle()
+        return
+
+    def set_text(self, markup):
+        gtk.Label.set_markup(self, '<span color="white">%s</span>' % markup)
+        return
+
+    def set_markup(self, markup):
+        gtk.Label.set_markup(self, '<span color="white">%s</span>' % markup)
+        return
+
+    def draw(self, cr, a, event_area):
+        if not self.get_property('visible'): return
+
+        cr.save()
+        xp = self.get_padding()[0]
+        ax, ay = self.get_alignment()
+        lx, ly, lw, lh = self.get_layout().get_pixel_extents()[1]
+        x = int(a.x + (a.width-lw)*ax)
+        y = int(a.y + (a.height-lh)*ay)
+        self.shape.layout(cr, x, y, x+lw+2*xp, y+lh, radius=3)
+        cr.set_source_rgba(0, 0, 0, 0.55)
+        cr.fill()
+        cr.restore()
+        return
+
+
+class MoreLabel(gtk.EventBox):
+
+    def __init__(self):
+        gtk.EventBox.__init__(self)
+        self.set_visible_window(False)
+
+        self.label = l = EtchedLabel()
+        l.set_alignment(0,0.5)
+        l.set_padding(6, 4)
+
+        self.add(l)
+
+        self.theme = get_mkit_theme()
+        self.shape = SHAPE_RECTANGLE
+
+        self.ishover = False
+        return
+
+    def set_text(self, markup):
+        gtk.Label.set_markup(self.label, '<small>%s</small>' % markup)
+        return
+
+    def set_markup(self, markup):
+        gtk.Label.set_markup(self.label, '<small>%s</small>' % markup)
+        return
+
+    def draw(self, cr, a, event_area):
+        if not self.get_property('visible'): return True
+
+        if self.state == gtk.STATE_PRELIGHT:
+            self.set_state(gtk.STATE_NORMAL)
+        self.theme.paint_bg(cr, self, a.x, a.y+2, a.width, a.height-4)
+        return
+
