@@ -14,35 +14,19 @@ import xapian
 from softwarecenter.db.application import Application, AppDetails
 from softwarecenter.db.database import StoreDatabase
 from softwarecenter.db.database import parse_axi_values_file
-from softwarecenter.db.update import update_from_app_install_data, update_from_var_lib_apt_lists
+from softwarecenter.db.update import update_from_app_install_data, update_from_var_lib_apt_lists, update_from_appstream_xml
 from softwarecenter.apt.aptcache import AptCache
+from softwarecenter.paths import *
 from softwarecenter.enums import *
 
 class TestDatabase(unittest.TestCase):
     """ tests the store database """
 
     def setUp(self):
-        apt_pkg.config.set("APT::Architecture", "i386")
         apt_pkg.config.set("Dir::State::status",
                            "./data/appdetails/var/lib/dpkg/status")
         self.cache = AptCache()
-
-    def test_comma_seperation(self):
-        xapian_base_path = XAPIAN_BASE_PATH
-        pathname = os.path.join(xapian_base_path, "xapian")
-        db = StoreDatabase(pathname, self.cache)
-        # normal
-        querries = db._comma_expansion("apt,2vcard,7zip")
-        self.assertEqual(len(querries), 3)
-        # multiple identical
-        querries = db._comma_expansion("apt,apt,apt")
-        self.assertEqual(len(querries), 1)
-        # too many commas
-        querries = db._comma_expansion(",,,apt,xxx,,,")
-        self.assertEqual(len(querries), 2)
-        # invalid query
-        querries = db._comma_expansion("??")
-        self.assertEqual(querries, None)
+        self.cache.open()
 
     def test_update_from_desktop_file(self):
         # ensure we index with german locales to test i18n
@@ -58,6 +42,20 @@ class TestDatabase(unittest.TestCase):
             i+=1
         self.assertEqual(i, 1)
 
+    def test_update_from_appstream_xml(self):
+        db = xapian.WritableDatabase("./data/test.db", 
+                                     xapian.DB_CREATE_OR_OVERWRITE)
+        res = update_from_appstream_xml(db, self.cache, "./data/app-info/")
+        self.assertTrue(res)
+        self.assertEqual(db.get_doccount(), 1)
+        # FIXME: improve tests
+        for p in db.postlist(""):
+            doc = db.get_document(p.docid)
+            for term in doc.termlist():
+                print term, term.term
+            for value in doc.values():
+                print value, value.num, value.value
+
     def test_update_from_var_lib_apt_lists(self):
         # ensure we index with german locales to test i18n
         os.environ["LANGUAGE"] = "de"
@@ -166,8 +164,8 @@ class TestDatabase(unittest.TestCase):
         # FIXME: add document that has a price
         self.assertEqual(appdetails.price, '')
         self.assertEqual(appdetails.license, "Open source")
-        # FIXME: this will only work if software-center is installed
-        self.assertNotEqual(appdetails.installation_date, None)
+        # test lazy history loading for installation date
+        self.ensure_installation_date_and_lazy_history_loading(appdetails)
         # test apturl replacements
         # $kernel
         app = Application("", "linux-headers-$kernel", "channel=$distro-partner")
@@ -178,6 +176,20 @@ class TestDatabase(unittest.TestCase):
         distro = get_distro().get_codename()
         self.assertEqual(app.request, 'channel=' + distro + '-partner')
         
+    def ensure_installation_date_and_lazy_history_loading(self, appdetails):
+        # we run two tests, the first is to ensure that we get a 
+        # result from installation_data immediately (at this point the
+        # history is not loaded yet) so we expect "None"
+        self.assertEqual(appdetails.installation_date, None)
+        # then we need to wait until the history is loaded in the idle
+        # handler
+        import gtk
+        while gtk.events_pending():
+            gtk.main_iteration()
+        # ... and finally we test that its really there
+        # FIXME: this will only work if software-center is installed
+        self.assertNotEqual(appdetails.installation_date, None)
+
     def test_package_states(self):
         db = xapian.WritableDatabase("./data/test.db", 
                                      xapian.DB_CREATE_OR_OVERWRITE)
@@ -209,6 +221,13 @@ class TestDatabase(unittest.TestCase):
         appdetails = app.get_details(db)
         self.assertEqual(appdetails.pkg_state, PKG_STATE_NOT_FOUND)
 
+    def test_packagename_is_application(self):
+        db = StoreDatabase("/var/cache/software-center/xapian", self.cache)
+        db.open()
+        # apt has no app
+        self.assertEqual(db.get_apps_for_pkgname("apt"), set())
+        # but software-center has
+        self.assertEqual(len(db.get_apps_for_pkgname("software-center")), 1)
 
     def test_whats_new(self):
         db = StoreDatabase("/var/cache/software-center/xapian", self.cache)
@@ -264,12 +283,12 @@ packagesize	3	# package size
 app-popcon	4	# app-install .desktop popcon rank
 """
         open("axi-test-values","w").write(s)
-        db = StoreDatabase("/var/cache/software-center/xapian", 
-                           self.cache)
+        db = StoreDatabase("/var/cache/software-center/xapian", self.cache)
         axi_values = parse_axi_values_file("axi-test-values")
         self.assertNotEqual(axi_values, {})
         print axi_values
 
+
 if __name__ == "__main__":
     import logging
     logging.basicConfig(level=logging.DEBUG)
